@@ -196,6 +196,20 @@ spin_changed (GtkSpinButton * spin, struct ALL_DATA *all_data)
 	videoIn = NULL;
 }
 
+/*set video frame jpeg quality/compression*/
+void
+set_jpeg_comp_clicked (GtkButton * jpeg_comp, struct ALL_DATA *all_data)
+{
+	struct GWIDGET *gwidget = all_data->gwidget;
+	struct vdIn *videoIn = all_data->videoIn;
+
+	videoIn->jpgcomp.quality = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(gwidget->jpeg_comp));
+
+	videoIn->setJPEGCOMP = 1; 
+	
+	videoIn = NULL;
+}
+
 /*check box controls callback*/
 void
 autofocus_changed (GtkToggleButton * toggle, struct ALL_DATA *all_data) 
@@ -997,7 +1011,10 @@ capture_vid (GtkToggleButton *VidButt, struct ALL_DATA *all_data)
 	{	/****************** Stop Video ************************/
 		closeVideoFile(all_data);
 		if(!(state))
+		{
 			gtk_button_set_label(GTK_BUTTON(gwidget->CapVidButt),_("Cap. Video"));
+			gtk_widget_show (gwidget->VidButton_Img);
+		}
 	} 
 	else if(!(videoIn->capVid) /*&& state*/)
 	{	/******************** Start Video *********************/
@@ -1025,7 +1042,10 @@ capture_vid (GtkToggleButton *VidButt, struct ALL_DATA *all_data)
 			state = FALSE;
 		}
 		if(state)
+		{
 			gtk_button_set_label(GTK_BUTTON(gwidget->CapVidButt),_("Stop Video"));
+			gtk_widget_show (gwidget->VidButton_Img);
+		}
 	}
 	
 	gwidget = NULL;
