@@ -28,7 +28,9 @@
 #include <glib/gprintf.h>
 /* support for internationalization - i18n */
 #include <glib/gi18n.h>
-#include <locale.h> //(gentoo patch) should already be included from libintl.h on glib/gi18n.h
+/* locale.h is needed if -O0 used (no optimiztions)  */
+/* otherwise included from libintl.h on glib/gi18n.h */
+#include <locale.h> 
 #include <gtk/gtk.h>
 #include <portaudio.h>
 
@@ -79,6 +81,7 @@ int main(int argc, char *argv[])
 {
 	int ret=0;
 	int n=0; //button box labels column
+	gboolean control_only = FALSE;
 	/*print package name and version*/ 
 	g_printf("%s\n", PACKAGE_STRING);
 	
@@ -116,6 +119,7 @@ int main(int argc, char *argv[])
 	/*------------------------- reads configuration file ---------------------*/
 	readConf(global);
 	
+	control_only = global->control_only; //sets local control_only flag
 	/*---------------------------------- Allocations -------------------------*/
 	
 	gwidget = g_new0(struct GWIDGET, 1);
@@ -140,7 +144,7 @@ int main(int argc, char *argv[])
 
 	s = g_new0(struct VidState, 1);
 
-	if(!(global->control_only)) /*control_only exclusion (video and Audio) */
+	if(!control_only) /*control_only exclusion (video and Audio) */
 	{
 		pdata = g_new0(struct paRecordData, 1);
 
@@ -201,7 +205,7 @@ int main(int argc, char *argv[])
 		{
 			case VDIN_DEVICE_ERR://can't open device
 				ERR_DIALOG (N_("Guvcview error:\n\nUnable to open device"),
-					N_("Please make sure the camera is connected\nand that the linux-UVC driver is installed."),
+					N_("Please make sure the camera is connected\nand that the correct driver is installed."),
 					&all_data);
 				break;
 				
@@ -227,8 +231,8 @@ int main(int argc, char *argv[])
 				else 
 				{
 					g_printerr("ERROR: Can't set video stream. No supported format found\nExiting...\n");
-					ERR_DIALOG (N_("Guvcview error:\n\nCan't set MJPG or YUV stream for guvcview"),
-						N_("Make sure you have a UVC compliant camera\nand that you have the linux UVC driver installed."),
+					ERR_DIALOG (N_("Guvcview error:\n\nCan't set a valid video stream for guvcview"),
+						N_("Make sure your device driver is v4l2 compliant\nand that it is properly installed."),
 						&all_data);
 				}
 				
@@ -241,6 +245,17 @@ int main(int argc, char *argv[])
 						&all_data);
 				}
 				break;
+
+			case VDIN_QUERYCAP_ERR:
+				ERR_DIALOG (N_("Guvcview error:\n\nCouldn't query device capabilities"),
+					N_("Make sure the device driver suports v4l2."),
+					&all_data);
+				break;
+			case VDIN_READ_ERR:
+				ERR_DIALOG (N_("Guvcview error:\n\nRead method error"),
+					N_("Please try mmap instead (--capture_method=1)."),
+					&all_data);
+				break;
 			
 			case VDIN_REQBUFS_ERR:/*unable to allocate dequeue buffers or mem*/
 			case VDIN_ALLOC_ERR:
@@ -262,8 +277,8 @@ int main(int argc, char *argv[])
 	if(videoIn->listFormats->current_format < 0) 
 	{
 		g_printerr("ERROR: Can't set video stream. No supported format found\nExiting...\n");
-		ERR_DIALOG (N_("Guvcview error:\n\nCan't set MJPG or YUV stream for guvcview"),
-			N_("Make sure you have a UVC compliant camera\nand that you have the linux UVC driver installed."),
+		ERR_DIALOG (N_("Guvcview error:\n\nCan't set a valid video stream for guvcview"),
+			N_("Make sure your device driver is v4l2 compliant\nand that it is properly installed."),
 			&all_data);
 	}
 	/* Set jpeg encoder buffer size */
@@ -327,7 +342,7 @@ int main(int argc, char *argv[])
 	gtk_widget_show (buttons_table);
 	gtk_paned_add2(GTK_PANED(gwidget->boxv),buttons_table);
 	
-	if(!(global->control_only)) /*control_only exclusion (video and Audio) */
+	if(!control_only) /*control_only exclusion (video and Audio) */
 	{
 		capture_labels=gtk_label_new(_("Capture:"));
 		gtk_misc_set_alignment (GTK_MISC (capture_labels), 0.5, 0.5);
@@ -360,7 +375,7 @@ int main(int argc, char *argv[])
 	}
 	g_free(icon1path);
 	
-	if(!(global->control_only))/*control_only exclusion Image and video buttons*/
+	if(!control_only)/*control_only exclusion Image and video buttons*/
 	{
 		if(global->image_timer)
 		{	/*image auto capture*/
@@ -393,7 +408,9 @@ int main(int argc, char *argv[])
 		
 			gtk_button_set_image(GTK_BUTTON(gwidget->CapVidButt),gwidget->VidButton_Img);
 			gtk_button_set_image_position(GTK_BUTTON(gwidget->CapVidButt),GTK_POS_TOP);
+			gtk_widget_show (gwidget->VidButton_Img);
 		}
+		//else g_printf("couldn't load %s\n", pix1path);
 		gchar* pix2path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/camera.png",NULL);
 		if (g_file_test(pix2path,G_FILE_TEST_EXISTS)) 
 		{
@@ -402,6 +419,7 @@ int main(int argc, char *argv[])
 		
 			gtk_button_set_image(GTK_BUTTON(gwidget->CapImageButt),ImgButton_Img);
 			gtk_button_set_image_position(GTK_BUTTON(gwidget->CapImageButt),GTK_POS_TOP);
+			gtk_widget_show (ImgButton_Img);
 		}
 		g_free(pix1path);
 		g_free(pix2path);
@@ -425,6 +443,7 @@ int main(int argc, char *argv[])
 		
 		gtk_button_set_image(GTK_BUTTON(SProfileButton),SButton_Img);
 		gtk_button_set_image_position(GTK_BUTTON(SProfileButton),GTK_POS_TOP);
+		gtk_widget_show (SButton_Img);
 	}
 	gchar* pix4path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/controls_folder.png",NULL);
 	if (g_file_test(pix4path,G_FILE_TEST_EXISTS)) 
@@ -434,6 +453,7 @@ int main(int argc, char *argv[])
 		
 		gtk_button_set_image(GTK_BUTTON(LProfileButton),LButton_Img);
 		gtk_button_set_image_position(GTK_BUTTON(LProfileButton),GTK_POS_TOP);
+		gtk_widget_show (LButton_Img);
 	}
 	gchar* pix5path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/close.png",NULL);
 	if (g_file_test(pix5path,G_FILE_TEST_EXISTS)) 
@@ -443,6 +463,7 @@ int main(int argc, char *argv[])
 		
 		gtk_button_set_image(GTK_BUTTON(quitButton),QButton_Img);
 		gtk_button_set_image_position(GTK_BUTTON(quitButton),GTK_POS_TOP);
+		gtk_widget_show (QButton_Img);
 	}
 
 	/*must free path strings*/
@@ -454,9 +475,9 @@ int main(int argc, char *argv[])
 	gtk_box_pack_start(GTK_BOX(HButtonBox),LProfileButton,TRUE,TRUE,2);
 	gtk_box_pack_start(GTK_BOX(HButtonBox),quitButton,TRUE,TRUE,2);
 
-	gtk_widget_show (LProfileButton);
-	gtk_widget_show (SProfileButton);
-	gtk_widget_show (quitButton);
+	gtk_widget_show_all (LProfileButton);
+	gtk_widget_show_all (SProfileButton);
+	gtk_widget_show_all (quitButton);
 
 	g_signal_connect (GTK_BUTTON(quitButton), "clicked",
 		G_CALLBACK (quitButton_clicked), &all_data);
@@ -477,7 +498,7 @@ int main(int argc, char *argv[])
 	}
 	gtk_paned_set_position (GTK_PANED(gwidget->boxv),global->boxvsize);
 	
-	if(!(global->control_only)) /*control_only exclusion (video and Audio) */
+	if(!control_only) /*control_only exclusion (video and Audio) */
 	{
 		/*------------------------- Video Tab ---------------------------------*/
 		video_tab (&all_data);
@@ -490,7 +511,7 @@ int main(int argc, char *argv[])
 	
 	gtk_widget_show (gwidget->mainwin);
 	
-	if (!(global->control_only)) /*control_only exclusion*/
+	if (!control_only) /*control_only exclusion*/
 	{
 		/* if autofocus exists allocate data*/
 		if(global->AFcontrol) 
@@ -555,7 +576,17 @@ int main(int argc, char *argv[])
 	gtk_main();
 	gdk_threads_leave();
 	
-	g_printf("Exit: OK\n");
+	//closing portaudio
+	if(!control_only)
+	{
+		g_printf("Closing portaudio ...");
+		if (Pa_Terminate() != paNoError) 
+			g_printf("Error\n");
+		else
+			g_printf("OK\n");
+	}
+	
+	//g_printf("Exit: OK\n");
 	return 0;
 }
 
