@@ -53,6 +53,7 @@
 #define VDIN_QUERYBUF_ERR        -11
 #define VDIN_QBUF_ERR            -12
 #define VDIN_MMAP_ERR            -13
+#define VDIN_READ_ERR            -14
 
 enum  v4l2_uvc_exposure_auto_type 
 {
@@ -74,13 +75,15 @@ struct vdIn
 {
 	GMutex *mutex;                      // VdIn struct mutex
 	int fd;                             // device file descriptor
-	char *videodevice;                  // video device string (default "/dev/video0"
+	char *videodevice;                  // video device string (default "/dev/video0)"
+	int cap_meth;                       // capture method : IO_MMAP (1)  IO_READ (0)
 	struct v4l2_capability cap;         // v4l2 capability struct
 	struct v4l2_format fmt;             // v4l2 formar struct
 	struct v4l2_buffer buf;             // v4l2 buffer struct
 	struct v4l2_requestbuffers rb;      // v4l2 request buffers struct
 	struct v4l2_timecode timecode;      // v4l2 timecode struct
 	struct v4l2_streamparm streamparm;  // v4l2 stream parameters struct
+	struct v4l2_jpegcompression jpgcomp;// v4l2 jpeg compression settings
 	void *mem[NB_BUFFER];               // memory buffers for mmap driver frames 
 	unsigned char *tmpbuffer;           // temp buffer for decoding compressed data
 	unsigned char *framebuffer;         // frame buffer (YUYV), for rendering in SDL overlay
@@ -88,6 +91,7 @@ struct vdIn
 	int isbayer;                        // raw bayer flag
 	int pix_order;                      // raw bayer pixel order (rg/gb, bg/gr, ...)
 	int setFPS;                         // show new FPS value flag
+	int setJPEGCOMP;                    //set jpeg compression flag
 	int grabmethod;                     // only mmap available UVC doesn't support read
 	int width;                          // frame width
 	int height;                         // frame height
